"""Mapping between icon numbers and their string representations"""

ICONS = {1: {'weather_id': 'not_available', 'weather': 'Not available'},
         2: {'weather_id': 'sunny', 'weather': 'Sunny'},
         3: {'weather_id': 'mostly_sunny', 'weather': 'Mostly sunny'},
         4: {'weather_id': 'partly_sunny', 'weather': 'Partly sunny'},
         5: {'weather_id': 'mostly_cloudy', 'weather': 'Mostly cloudy'},
         6: {'weather_id': 'cloudy', 'weather': 'Cloudy'},
         7: {'weather_id': 'overcast', 'weather': 'Overcast'},
         8: {'weather_id': 'overcast_w_low_clds', 'weather': 'Overcast with low clouds'},
         9: {'weather_id': 'fog', 'weather': 'Fog'},
         10: {'weather_id': 'light_rain', 'weather': 'Light rain'},
         11: {'weather_id': 'rain', 'weather': 'Rain'},
         12: {'weather_id': 'psbl_rain', 'weather': 'Possible rain'},
         13: {'weather_id': 'rain_shower', 'weather': 'Rain shower'},
         14: {'weather_id': 'tstorm', 'weather': 'Thunderstorm'},
         15: {'weather_id': 'tstorm_shower', 'weather': 'Local thunderstorms'},
         16: {'weather_id': 'light_snow', 'weather': 'Light snow'},
         17: {'weather_id': 'snow', 'weather': 'Snow'},
         18: {'weather_id': 'psbl_snow', 'weather': 'Possible snow'},
         19: {'weather_id': 'snow_shower', 'weather': 'Snow shower'},
         20: {'weather_id': 'rain_and_snow', 'weather': 'Rain and snow'},
         21: {'weather_id': 'psbl_rain_and_snow', 'weather': 'Possible rain and snow'},
         22: {'weather_id': 'rain_and_snow', 'weather': 'Rain and snow'},
         23: {'weather_id': 'fr_rain', 'weather': 'Freezing rain'},
         24: {'weather_id': 'psbl_fr_rain', 'weather': 'Possible freezing rain'},
         25: {'weather_id': 'hail', 'weather': 'Hail'},
         26: {'weather_id': 'clear', 'weather': 'Clear'},
         27: {'weather_id': 'mostly_clear', 'weather': 'Mostly clear'},
         28: {'weather_id': 'partly_clear', 'weather': 'Partly clear'},
         29: {'weather_id': 'mostly_cloudy', 'weather': 'Mostly cloudy'},
         30: {'weather_id': 'cloudy', 'weather': 'Cloudy'},
         31: {'weather_id': 'overcast_w_low_clds', 'weather': 'Overcast with low clouds'},
         32: {'weather_id': 'rain_shower', 'weather': 'Rain shower'},
         33: {'weather_id': 'tstorm_shower', 'weather': 'Local thunderstorms'},
         34: {'weather_id': 'snow_shower', 'weather': 'Snow shower'},
         35: {'weather_id': 'rain_and_snow', 'weather': 'Rain and snow'},
         36: {'weather_id': 'psbl_fr_rain', 'weather': 'Possible freezing rain'},
}
